#!/usr/bin/perl -w
# 
# sb::Plugin::LimeCounter
# == written by Assy <info@aizack.net> ===
# == Copyright (C) 2021 AIZACK.NET ==

# ======================================
package sb::Plugin::LimeCounter;

use sb::Plugin ();

sb::Plugin->register_plugin(
'lang' => {
	'ja' => 'utf-8',
	'en' => 'ascii',
},
'text' => {
	'type'    => 'admin, cms',
	'name'    => 'LimeCounter',
	'text'    => 'LimeCounter plugin for Serene Bach',
	'author'  => 'Assy',
	'detail'  => 'http://aizack.net/',
	'version' => version(),
},
'file' => 'limecounter.txt',
'data' => 1,
);

sb::Plugin->register_admin_module(
'mode'   => 'limecounter',
'level'  => 1,
'module' => 'sb::Admin::LimeCounter',
);

sb::Plugin->register_content_module(
'type'	   => 'main',
'callback' => \&sb::Content::LimeCounter::_list,
'field'    => 'limecounter',
);

sub version
{
	return '0.30';
	# 0.30 [2021/02/23] 外部公開用にスタイルシートを指定出来る様に独自タグ{limecounter_css}を復活。
	#                   設定項目に外部公開用スタイルシートを編集出来るようにした。
	#					外部公開の場合、文字化けしていたバグを修正。
	#					設定チェックにスタイルシートファイルもチェックするようにした。
	# 0.20a [2021/01/19] SereneBachのEUC-JP版で文字化けしないように文字コード処理を施した。
	#					設置方法の文面を変更した。
	# 0.20 [2021/01/11] 文字コードがバラバラで処理していたのでUTF-8で統一し文字コード処理のコードを簡略化した。
	#					プラグイン本体のコードの不要部分を削除して簡略化した。
	#					管理画面の編集・削除・新規のテンプレートを変更した。
	#					管理画面の設定項目にデフォルト値を入れる事にした。
	#					独自タグ{limecounter_css}を一時的に使用不可とした。
	# 0.12b[2010/11/14] lime.cgiのチェックモード時もカウントアップを行っていたバグを修正した。
	# 0.12a[2010/01/31] 必要なタグが出力されない不具合を修正した。
	# 0.12 [2009/12/05] HTMLの文法チェッカでの減点対象を減らした。
	# 0.11 [2009/11/25] アイテムを削除できないバグを修正しました。
	#                   lime.cgiの動作チェックを管理画面に追加しました。
	# 0.10 [2009/11/20] オリジナルのLimeCounterをインストールしなくても動作するように変更。
	# 0.00 [2009/07/16] 初公開
	
}

# ======================================
# コンテンツ用プラグイン設定
# ======================================
package sb::Content::LimeCounter;

use sb::Language ();
use sb::TemplateManager ();
use sb::Config ();

#-------------------------------------------------
# callbackサブルーチン
#-------------------------------------------------
sub _list {
	my $cms = shift;
	my %var = @_;
	my $list = &sb::Admin::LimeCounter::_list_data();
	$cms->num(0);
	$cms->tag('limecounter_list' => $list);
	my $data = sb::Plugin->get_data;
	my $datadir = $data->data;
	$cms->tag('limecounter_css' => '<link rel="stylesheet" type="text/css" href="' . $datadir . '/limecounter.css" />');
	return 1;
}
# ======================================

# ======================================
# 管理用プラグイン設定
# ======================================
package sb::Admin::LimeCounter;
use vars qw( @ISA );
use sb::Language ();
use sb::Interface ();
use sb::TemplateManager ();
use sb::Text ();
use sb::App::Admin ();
use sb::Config ();

@ISA = qw( sb::App::Admin );
# --------------------------------------
sub TEMPLATE (){ 'limecounter.html' }; #管理用テンプレートファイル
sub PARTS_BAR(){ 'accesslog_bar.gif' }; #集計リストグラフ画像
sub CSS_FILE(){ 'limecounter.css' }; #スタイルシートファイル

#データディレクトリ
sub DATA_DIR(){
	my $data = sb::Plugin->get_data;
	return $data->data;
};
#インデックスファイル
sub IDX_FILE(){
	my $data = sb::Plugin->get_data;
	return $data->data . "/" . $data->text;
};

#-------------------------------------------------
#文字コード変換(SBの文字コードへ)
#-------------------------------------------------
sub CHAR_CONVERT {
	local($chr) = @_;
	my $code = sb::Language->get->charcode;
	sb::Language->get->checkcode('','utf-8');
	return sb::Language->get->convert($chr,$code);
};

#-------------------------------------------------
#文字コード変換(インデックスファイルにutf8で保存する為)
#-------------------------------------------------
sub UTF8_CONVERT {
	local($chr) = @_;
	my $code = sb::Language->get->charcode;
	sb::Language->get->checkcode('',$code);
	return sb::Language->get->convert($chr,'utf8');
};

#-------------------------------------------------
#  桁区切り
#-------------------------------------------------
sub comma {
	local($_) = @_;
	
	1 while s/(.*\d)(\d\d\d)/$1,$2/;
	$_;
};

# --------------------------------------

sub callback
{
	my $self = shift;
	return ( $self->{'regi'} ) 
	? $self->_save_data(@_)
	: $self->_open_data(@_);
};

#-------------------------------------------------
#  集計データ保存（インデックスファイルに保存）
#-------------------------------------------------
sub _save_data
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi  = sb::Interface->get;
	my $cms  = sb::TemplateManager->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my $data = sb::Plugin->get_data;
	my $type = $cgi->value('__type');
	my $job = $cgi->value('__job');
	
	#設定項目の保存
	if ($type eq 'setting')	{
		#ディレクトリ確認
		if ($cgi->value('limecounter_dir') eq ''){
			$data->data(sb::Text->entitize('lime'));#空欄の場合、limeディレクトリを使用
		}
		else{
			$data->data(sb::Text->entitize($cgi->value('limecounter_dir')));
		}
		#インデックスファイル確認
		if ($cgi->value('limecounter_idx') eq ''){
			$data->text(sb::Text->entitize('lime.log'));#空欄の場合、lime.logを使用
		}
		else{
			$data->text(sb::Text->entitize($cgi->value('limecounter_idx')));
		}
		#ディレクトリとインデックスファイルをplugin.cgiに保存
		sb::Plugin->set_data('data'=>$data);
		
		#CSSをファイルに保存
		if ($cgi->value('limecounter_extra_css') ne '')
		{
			my $file = DATA_DIR . "/" . CSS_FILE;
			open(OUT,">$file");
			binmode(OUT);
			print OUT $cgi->value('limecounter_extra_css');
			close(OUT);
		}		
		
	}
	
	#新規追加
	elsif ($type eq 'new') {
		my $in_id   = $cgi->value('__id');
		my $in_sub  = UTF8_CONVERT($cgi->value('__sub'));
		my $in_count = $cgi->value('__count');
		my $in_file = $cgi->value('__file');
		my $in_link = $cgi->value('__link');
		
		if ($in_file eq "http://") { $in_file = ""; }
		if ($in_link eq "http://") { $in_link = ""; }
		if ($in_id =~ /\W/) {return $self->_open_data('message'=>sb::Language->get->string('item_checkidnum'));}
		if ($in_id eq "check") {return $self->_open_data('message'=>sb::Language->get->string('item_checkidname'));}
		if ($in_id eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checkid'));}
		if ($in_sub eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checktitle'));}
		if ($in_file eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checkfile'));}
		if ($in_count eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checkcount'));}
		
		my $errormsg = &_newitem($in_id,$in_sub,$in_count,$in_link,$in_file);
		
		if ($errormsg ne "")	{return $self->_open_data('message'=>$errormsg);}
		return $self->_open_data('message'=>sb::Language->get->string('item_new'));
	}
	
	#編集
	elsif ($type eq 'edit') {
		#削除
		if ($job eq 'dele') {
			my $in_id = $cgi->value('radio_id');
			if ($in_id eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checkdel'));}
			&_deleteitem($in_id);
			return $self->_open_data('message'=>sb::Language->get->string('item_delete'));
		}
		#編集画面
		elsif ($job eq 'edit2') {
			my $in_id = $cgi->value('radio_id');
			if ($in_id eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checkedit'));}
			return $self->_open_data('radio_id'=>$in_id);
		}
		#編集実行
		elsif ($job eq 'edit3') {
			my $in_id   = $cgi->value('__id');
			my $in_sub  = UTF8_CONVERT($cgi->value('__sub'));
			my $in_count = $cgi->value('__count');
			my $in_pcount = $cgi->value('__pcount');
			my $in_file = $cgi->value('__file');
			my $in_link = $cgi->value('__link');
			
			if ($in_file eq "http://") { $in_file = ""; }
			if ($in_link eq "http://") { $in_link = ""; }
			if ($in_sub eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checktitle'));}
			if ($in_file eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checkfile'));}
			if ($in_count eq "") {return $self->_open_data('message'=>sb::Language->get->string('item_checkcount'));}
			
			my $errormsg = &_edititem($in_id,$in_sub,$in_count,$in_pcount,$in_link,$in_file);
			
			if ($errormsg ne "")	{return $self->_open_data('message'=>$errormsg);}
			return $self->_open_data('message'=>sb::Language->get->string('item_edit'));
		}
	}
	return $self->_open_data('message'=>sb::Language->get->string('parts_confcomp'));
}

#-------------------------------------------------
#  集計データ読み込み（インデックスファイルから読み込み）
#-------------------------------------------------
sub _open_data
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi  = sb::Interface->get;
	my $cms  = sb::TemplateManager->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my $data = sb::Plugin->get_data;
	my $type = $cgi->value('__type');
	my $job = $cgi->value('__job');
	
	#	$self->common_template_parts($cms);
	my $datadir = $data->data;
	my $idxfile = $data->text;
	$type = 'display' if ($type ne 'setting' and $type ne 'edit' and $type ne 'new');
	$cms->num(0);
	
	# display mode
	if ($type eq 'display')
	{
		my $down = &_list_data;
		$cms->tag('limecounter_list' => $down);
		$cms->tag('sb_limecounter_menu_display'=>'class="current"');
		$cms->block('sb_limecounter_display'=>1);
	}
	# end of display mode
	
	# setting mode
	elsif ($type eq 'setting')
	{
		if($datadir eq ''){$datadir = 'lime'}
		if($idxfile eq ''){$idxfile ='lime.log'}
		$cms->tag('sb_dir_base' => sb::Config->get->value('conf_srv_base'));
		$cms->tag('limecounter_dir' => $datadir);
		$cms->tag('limecounter_idx' => $idxfile);
		$cms->tag('limecounter_extra_css'=>$self->_get_style());		
		$cms->tag('sb_limecounter_menu_setting'=>'class="current"');
		$cms->block('sb_limecounter_setting'=>1);
	}
	# end of setting mode
	
	# edit mode
	elsif ($type eq 'edit')
	{
		#編集画面
		if ($job eq 'edit2') {
			if ($param{'message'} ne sb::Language->get->string('item_checkedit')){
				my $in_id = $param{'radio_id'};
				# indexチェック
				open(IN,"$datadir/$idxfile");
				while (<IN>) {
					($edit_id,$edit_sub,$edit_link,$edit_file) = split(/<>/);
					if ($in_id eq $edit_id) { last; }
				}
				close(IN);
				#カウンタ読込
				open(DB,"$datadir/$edit_id.dat");
				my $count = <DB>;
				close (DB);
				my $edit_count;
				my $ip;
				($edit_count,$ip) = split(/:/, $count);
				$cms->tag('limecounter_id' => $edit_id);
				$cms->tag('limecounter_sub' => CHAR_CONVERT($edit_sub));
				$cms->tag('limecounter_count' => $edit_count);
				$cms->tag('limecounter_file' => $edit_file);
				$cms->tag('limecounter_link' => $edit_link);
				$cms->tag('sb_limecounter_menu_edit'=>'class="current"');
				$cms->block('sb_limecounter_editform'=>1);
			}
			else {
				my $editlist = &_editlist;
				$cms->tag('limecounter_edit' => $editlist);
				$cms->tag('sb_limecounter_menu_edit'=>'class="current"');
				$cms->block('sb_limecounter_edit'=>1);
			}
		}
		#編集リスト
		else {
			my $editlist = &_editlist;
			$cms->tag('limecounter_edit' => $editlist);
			$cms->tag('sb_limecounter_menu_edit'=>'class="current"');
			$cms->block('sb_limecounter_edit'=>1);
		}
	}
	# end of edit mode
	
	# delete mode(削除後)
	elsif ($type eq 'dele')
	{
		my $editlist = &_editlist;
		$cms->tag('limecounter_edit' => $editlist);
		$cms->tag('sb_limecounter_menu_edit'=>'class="current"');
		$cms->block('sb_limecounter_edit'=>1);
	}
	# end of delete mode
	
	# new mode(新規)
	elsif ($type eq 'new')
	{
		$cms->tag('sb_limecounter_menu_new'=>'class="current"');
		$cms->block('sb_limecounter_new'=>1);
	}
	# end of new mode
	
	if ($param{'message'} ne '')
	{
		$cms->tag('sb_process_message'=>$param{'message'});
		$cms->block('sb_process_message'=>1);
	}
	return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}

#スタイルシートファイルの中身を読み込む
sub _get_style
{
	my $self = shift;
	return $self->load_template(
	'dir'=>DATA_DIR . "/",
	'file'=>CSS_FILE
	);
}
#-------------------------------------------------
#  集計サブルーチン
#-------------------------------------------------
sub _list_data
{
	my $datadir = DATA_DIR;
	my $idxfile = IDX_FILE;	
	my $ver = &sb::Plugin::LimeCounter::version;
	my $barimg = sb::Plugin->get_resource_dir . PARTS_BAR; #集計リストグラフ画像	
	
	# 集計結果はソートする(0 : しない 1 : する)
	my $sort = 1;
	
	local($i,$r,$f,$bef);
	
	# データ読み込み
	my $all = 0;
	my (@data,@sort);
	my $downlist;
	
	open(IN,"$idxfile") || return $idxfile . " Open Error\n";
	while (<IN>) {
		my ($id,$sub,$link,$file) = split(/<>/);
		open(DB,"$datadir/$id.dat");
		my $data2 = <DB>;
		close(DB);
		
		# ログ分解
		my ($count, $ip) = split(/:/, $data2);
		
		# 総計
		$all += $count;
		
		push(@sort,$count) if ($sort);
		push(@data,"$id<>$sub<>$link<>$count");
	}
	close(IN);
	
	# ソート処理
	if ($sort) {
		@data = @data[sort {$sort[$b] <=> $sort[$a]} 0 .. $#sort];
	}
	$downlist .= qq|<div align="center">\n<table class="list">\n<tr class="lead">|;
	
	my $cols = 3;
	if ($sort) {
		$downlist .= qq|<th>順位</th>|;
		$cols++;
	}
	
	$downlist .= qq|<th>項目</th><th>カウント</th><th>統計</th></tr>|;
	
	# 集計テーブル
	my $i = 0;
	my $r = 1;
	my $f = 0;
	foreach (@data) {
		my ($id,$sub,$link,$count) = split(/<>/);
		$i++;
		
		my $per;
		if ($all > 0) {
			$per = sprintf("%.1f", $count*100/$all);
			if ($per > 100) { $per = 100; }
			$wid = int($per * 5);
			if ($wid < 1) { $wid = 1; }
		} else {
			$per = 0.0;
			$wid = 1;
		}
		$count = &comma($count);
		if ($count ne $bef) { $r = $i; }
		
		# 結果を表示
		if ($r % 2 == 0){
			$downlist .= qq|<tr class="even">|;
		}
		else {
			$downlist .= qq|<tr class="odd">|;
		}	
		if ($sort) {
			$downlist .= qq|<td>$r</td>|;
		}
		
		$downlist .= qq|<td class="elem">|;
		$downlist .= qq|<a href="$link" target="_blank"><strong>$sub</strong></a></td>|;
		$downlist .= qq|<td class="head">$count</td>|;
		$downlist .= qq|<td class="elem"><img src="$barimg" width="$wid" height="10" alt="" /> $per\%</td></tr>\n|;
		
		$bef = $count;
	}
	
	$all = &comma($all);
	
	# 合計
	$downlist .= qq|<tr class="odd">\n|;
	
	if ($sort) { $downlist .= qq|<td class="elem"></td>|; }
		
		$downlist .= <<EOM;
<td class="head">合計</td>
<td class="head">$all</td>
<td class="head"></td>
</tr>
</table>
</div>
<div align="center">
<span style="font-size:10px;font-family:Verdana,Helvetica,Arial">
- Powerd by <a href="http://www.kent-web.com/" target="_blank">LimeCounter</a> ver2.21-<br />
- Edit by <a href="http://aizack.net/" target="_blank">AIZACK.NET</a> -<br />
- LimeCounter Plugin v$ver -
</span>
</div>

EOM
		return  CHAR_CONVERT($downlist);
	}
	
	#-------------------------------------------------
	#  編集リスト
	#-------------------------------------------------
	sub _editlist {
		
		my $cgi = sb::App::Admin->get_script_path; #admin.cgiのフルパスを取得
		my $datadir = DATA_DIR;
		my $idxfile = IDX_FILE;
		my $editlist .= <<EOM;
<ul>
<li>処理を選択して実行ボタンを押してください。</li>
</ul>
<form action="$cgi" name="mainform" method="post">
処理： <select name="__job">
<option value="edit2">修正
<option value="dele">削除
</select>
<input type="hidden" name="__type" value="edit" />
<input type="hidden" name="__mode" value="limecounter" />
<input type="submit" name="__regi" value="&gt; 実行 &lt;" class="button" />
<table  cellpadding="0" cellspacing="1" class="list">
<tr class="lead">
	<th>選択</th>
	<th>ID名</th>
	<th>タイトル</th>
	<th>カウント</th>
</tr>
EOM
		
		# index展開
		open(IN,"$idxfile") || return $idxfile . " Open Error\n";
		while (<IN>) {
			my ($id,$sub,$link,$file,$cat) = split(/<>/);
			# カウンタ読み取り
			open(DB,"$datadir/$id.dat");
			my $count = <DB>;
			close(DB);
			my ($count,$ip) = split(/:/, $count);
			# 桁区切り
			$count = &comma($count);
			$editlist .= qq|<tr class="odd"><td><input type="radio" name="radio_id" value="$id"></td><td class="elem" align="left"><b>$id</b></td><td class="elem" align="left">$sub</td><td>$count</td></tr>\n|;
		}
		close(IN);
		
		$editlist .=  <<EOM;
</table>
</form>
EOM
		return  CHAR_CONVERT($editlist);
	}
	
	#-------------------------------------------------
	#  新規
	#-------------------------------------------------
	sub _newitem {
		
		local($newid,$newsub,$newcount,$newlink,$newfile) = @_;
		my $datadir = DATA_DIR;
		my $idxfile = IDX_FILE;	
		
		# indexチェック
		my ($flg,@data);
		open(DAT,"+< $idxfile") || return "Open Error: $idxfile";
		while (<DAT>) {
			my ($id) = split(/<>/);
			if ($newid eq $id) {
				$flg++;
				last;
			}
			push(@data,$_);
		}
		
		# 重複
		if ($flg) {
			close(DAT);
			return CHAR_CONVERT("$newidは既存のIDと重複しています");
		}
		
		# index更新
		seek(DAT, 0, 0);
		print DAT "$newid<>$newsub<>$newlink<>$newfile<>\n";
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);
		
		# カウンタ値
		open(OUT,">$datadir/$newid.dat") || return "Write Error: $newid.dat";
		print OUT "$newcount\n";
		close(OUT);
		return;
	}
	
	#-------------------------------------------------
	# 修正実行
	#-------------------------------------------------
	sub _edititem {
		
		local($editid,$editsub,$editcount,$pcount,$editlink,$editfile) = @_;
		my $datadir = DATA_DIR;
		my $idxfile = IDX_FILE;	
		
		# index差替え
		my @data;
		open(DAT,"+< $idxfile") || return "Open Error: $idxfile";
		while (<DAT>) {
			my ($id) = split(/<>/);
			if ($editid eq $id) {
				$_ = "$editid<>$editsub<>$editlink<>$editfile<>\n";
			}
			push(@data,$_);
		}
		seek(DAT, 0, 0);
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);
		
		# カウンタ更新
		if ($pcount != $editcount) {
			open(DAT,"+< $datadir/$editid.dat") || return "Write Error: $editid.dat";
			eval "flock(DAT, 2);";
			my $count = <DAT>;
			my ($count, $ip) = split(/:/, $count);
			seek(DAT, 0, 0);
			print DAT "$editcount:$ip";
			truncate(DAT, tell(DAT));
			close(DAT);
		}
		return;
	}
	
	
	#-------------------------------------------------
	#  削除
	#-------------------------------------------------
	sub _deleteitem {
		
		local($in_id) = @_;
		my $datadir = DATA_DIR;
		my $idxfile = IDX_FILE;	
		
		# indexマッチング
		my @data;
		open(DAT,"+< $idxfile") || return "Open Error: $idxfile";
		while (<DAT>) {
			my ($id) = split(/<>/);
			next if ($in_id eq $id);
			push(@data,$_);
		}
		# index更新
		seek(DAT, 0, 0);
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);
		
		# ログ削除
		unlink("$datadir/$in_id.dat");
	}
	
	# ======================================
	package sb::App::LimeCounter;
	
	use vars qw( @ISA );
	use sb::Language ();
	use sb::Interface ();
	use sb::Lock ();
	use sb::App ();
	use base sb::App;
	@ISA = qw( sb::App );
	
	# ---------------------------------------------------
	sub run {
		# 外部データ受け取り
		my $buf = $ENV{'QUERY_STRING'};
		$buf =~ s/\W//g;
		
		my $cgi   = sb::Interface->get;
		my $class = shift;
		my $self  = $class->SUPER::new(@_);
		my ($name) = (__FILE__ =~ m/([^\/]+$)/);
		if (&sb::Plugin::_ignore_plugin($name)) {
			$self->_response('plugin is not available.');
			return;
		}
		
		# チェックモード
		if (!$buf || $buf eq "check") { &_check; }
		
		# カウント処理
		else {	&_count_up($buf);
		}
	}
	
	sub _response {
		my $self = shift;
		my $error = shift;
		$error = ($error) ? "<failed>$error</failed>\n" : "<suceeded />\n";
		print sb::Interface->get->head('type'=>'text/xml');
		print '<?xml version="1.0" encoding="utf-8"?>' . "\n" . $error;
	}
	
	#-------------------------------------------------
	#  チェックモード
	#-------------------------------------------------
	sub _check {
		
		my $datadir = &sb::Admin::LimeCounter::DATA_DIR;
		my $idxfile = &sb::Admin::LimeCounter::IDX_FILE;
		my $cssfile = $datadir . "/" . &sb::Admin::LimeCounter::CSS_FILE;
		my $ver = &sb::Plugin::LimeCounter::version;
		my $cgi   = sb::Interface->get;
		my $output;
		$output .= <<EOM;
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
$meta
<title>LimeCounter Plugin ver$ver</title>
</head>
<body>
<h3>Check Mode</h3>
<ul>
EOM
		
		# indexのパス確認
		if (-e $idxfile) {
			$output .= "<li>インデックスファイルのパス: OK!</li>\n";
			
			# パーミッション
			if (-r $idxfile && -w $idxfile) {
				$output .= "<li>インデックスファイルのパーミッション: OK!</li>\n";
			} 
			else {
				$output .= "<li>インデックスファイルのパーミッション: NG → $idxfile</li>\n";
			}
		}
		else {
			$output .= "<li>インデックスファイルのパス: NG → $idxfile</li>\n";
		}
		
		# cssfileのパス確認
		if (-e $cssfile) {
			$output .= "<li>スタイルシートのパス: OK!</li>\n";
			# パーミッション
			if (-r $cssfile && -w $cssfile) {
				$output .= "<li>スタイルシートのパーミッション: OK!</li>\n";
			} 
			else {
				$output .= "<li>スタイルシートのパーミッション: NG → $cssfile</li>\n";
			}
		}
		else {
			$output .= "<li>スタイルシートのパス: NG → $cssfile</li>\n";
		}
		
		# データディレクトリ
		if (-d $datadir) {
			$output .= "<li>データディレクトリのパス: OK!</li>\n";
			
			if (-r $datadir && -w $datadir && -x $datadir) {
				$output .= "<li>データディレクトリのパーミッション: OK!</li></ul>\n";
			} 
			else {
				$output .= "<li>データディレクトリのパーミッション: NG → $datadir</li></ul>\n";
			}
		}
		else {
			$output .= "<li>データディレクトリのパス: NG → $datadir</li></ul>\n";
		}
		
		
		$output .= "<ul><li>LimeCounter Plugin for SereneBach バージョン: $ver</li>\n";
		# 著作権表示：削除禁止
		$output .= <<EOM;
<li>Original Script バージョン: <a href="http://www.kent-web.com/">LimeCounter v2.21</a></li>
</ul>
</body>
</html>
EOM
		print $cgi->head('type'=>'text/html','charset'=>'utf-8','length'=>length($output)) . $output;
	}
	
	#-------------------------------------------------
	#  カウント処理
	#-------------------------------------------------
	sub _count_up {
		
		#IPチェックによる重複カウント防止 (0=no 1=yes)
		$ip_chk = 1;
		
		my $buf = $_[0];
		
		# ファイル定義
		my $datadir = &sb::Admin::LimeCounter::DATA_DIR;
		my $datafile = "$datadir/$buf.dat";
		my $idxfile = &sb::Admin::LimeCounter::IDX_FILE;
		
		# データが存在しない場合は終了
		unless (-e $datafile) { return $datafile . " Not Exist\n"; }
		
		# IPアドレス取得
		my $addr = $ENV{'REMOTE_ADDR'};
		
		# データ読み取り
		open(DAT,"+< $datafile") || return $datafile . " Open Error\n";
		eval "flock(DAT, 2);";
		my $data = <DAT>;
		
		# データ分解
		my ($count, $ip) = split(/:/, $data);
		
		# カウントアップ
		if (!$ip_chk || ($ip_chk && $addr ne $ip)) {
			$count++;
			seek(DAT, 0, 0);
			print DAT "$count:$addr";
			truncate(DAT, tell(DAT));
		}
		close(DAT);
		
		# index読み取り
		my ($flg, $jump);
		open(IN,"$idxfile");
		while (<IN>) {
			my ($id,$sub,$link,$file) = split(/<>/);
			
			if ($buf eq $id) {
				$flg++;
				$jump = $file;
				last;
			}
		}
		close(IN);
		
		if (!$flg) { return "IDが不正です"; }
		
		# IISサーバ (PerlIS) 対応
		if ($ENV{'PERLXS'} eq "PerlIS") {
			print "HTTP/1.0 302 Temporary Redirection\r\n";
			print "Content-type: text/html\n";
		}
		
		# ファイルへジャンプ
		print "Location: $jump\n\n";
	}
	
	1;
	__END__
